"
 COPYRIGHT (c) Claus Gittinger / eXept Software AG
 COPYRIGHT (c) 2015-2016 Jan Vrany
 COPYRIGHT (c) 2020 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libbasic3' }"

"{ NameSpace: Smalltalk }"

AutomagicSourcePorter subclass:#BeeSourcePorter
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Classes-Support-Autoporting'
!

!BeeSourcePorter class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) Claus Gittinger / eXept Software AG
 COPYRIGHT (c) 2015-2016 Jan Vrany
 COPYRIGHT (c) 2020 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    BeeSourcePorter is a utility class used to automagically
    port (method) code from Smalltalk/X to Bee Smalltalk. This
    is done by automatically rewriting method source code when
    writing Bee project files.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!BeeSourcePorter class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    "/ please change as required (and remove this comment)

    defaultKlassMap := Dictionary new.
    defaultKlassMap at: #ApplicationModel put: #CompositePane

    "Modified: / 29-10-2018 / 15:55:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 01-07-2020 / 12:29:22 / Jan Vrany <jan.vrany@labware.com>"
! !

!BeeSourcePorter methodsFor:'rewriting'!

rewrite
    super rewrite.

    self
        rewriteAnnotationsToComments; checkSource

    "Created: / 16-10-2018 / 20:29:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-11-2018 / 14:43:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 01-07-2020 / 12:49:25 / Jan Vrany <jan.vrany@labware.com>"
! !

!BeeSourcePorter methodsFor:'rewriting-individual'!

rewriteAnnotationsToComments
    |parser annots|

    parser := Parser new.
    parser foldConstants:false.
    parser saveComments:true.
    parser parseMethod:source in:klass ignoreErrors:true ignoreWarnings:true.

    (annots := parser annotationInfo) isEmptyOrNil ifTrue:[^ self].

    annots := annots copy sort:[:a :b | a endPosition < b endPosition].
    annots reverseDo:[:annot |
        source := (source copyTo:annot startPosition-1)
                  , '"' , (source copyFrom:annot startPosition to:annot endPosition)
                  , '"' , (source copyFrom:annot endPosition+1).
    ].

    "Created: / 16-10-2018 / 20:27:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

rewriteSimpleExpressions
    | replacer  tree |

    replacer := ParseTreeSourceRewriter new.
    replacer alwaysPerformAllSearches:true.
     
    "/ ? becomes ifNil:    
    replacer replace:'``@a ? ``@b' with:'``@a ifNil: ``@b'.
     
    "/ #isDigitRadix: becomes: #isDigitBase:    
    replacer replace:'``@a isDigitRadix: ``@b' with:'``@a isDigitBase: ``@b'.
     
    "/ #isLetterOrDigit or #isAlphaNumeric becomes #isAlphanumeric    
    replacer replace:'``@a isLetterOrDigit' with:'``@a isAlphanumeric'.
    replacer replace:'``@a isAlphaNumeric' with:'``@a isAlphanumeric'.
     
    "/ #asSymbolIfInterned becomes #asExistingSymbolOrNil    
    replacer replace:'``@a asSymbolIfInterned' with:'``@a asExistingSymbolOrNil'.


    replacer replace:'Smalltalk isStandAloneApp' with:'SmalltalkSession current isRuntime'.

    replacer replace:'Smalltalk isSmalltalkX' with:'false'.    

    "/ #assert: becomes #ASSERT: but only when in a class that is
    "/ not a TestCase!!
    (klass inheritsFrom: TestCase) ifFalse:[
        replacer replace:'``@a assert: ``@c' with:'``@a ASSERT: ``@c'.
    ].
    

    tree := RBParser parseMethod:source.
    replacer executeTree:tree.
    source := replacer executeReplacementsInSource:source.
    self checkSource

    "Created: / 29-10-2018 / 15:32:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 05-11-2018 / 22:36:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!BeeSourcePorter class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


BeeSourcePorter initialize!
